/**
 * Ultrasonic sensor uses NewPing library
 * https://bitbucket.org/teckel12/arduino-new-ping/wiki/Home
 * 
 * Display uses U8g2 library
 * https://github.com/olikraus/u8g2
 * 
 * Sounds from Active buzzer example on https://www.arduino.cc/en/Tutorial/PlayMelody
 */

#include <NewPing.h>

#define TRIGGER_PIN  4  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     5  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 300 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

///////////////// support for buzzer:
// TONES  ==========================================
// Start by defining the relationship between
//       note, period, &  frequency.
#define  c     3830    // 261 Hz
#define  d     3400    // 294 Hz
#define  e     3038    // 329 Hz
#define  f     2864    // 349 Hz
#define  g     2550    // 392 Hz
#define  a     2272    // 440 Hz
#define  b     2028    // 493 Hz
#define  C     1912    // 523 Hz
// Define a special note, 'R', to represent a rest
#define  R     0

// Set up speaker (buzzer) on a PWM pin (digital 9, 10 or 11)
#define speakerOut 9

// MELODY and TIMING  =======================================
//  melody[] is an array of notes, accompanied by beats[],
//  which sets each note's relative length (higher #, longer note)
int melody[] = {  C,  b,  g,  C,  b,   e,  R,  C,  c,  g, a, C };
int beats[]  = { 16, 16, 16,  8,  8,  16, 32, 16, 16, 16, 8, 8 };
int MAX_COUNT = sizeof(melody) / 2; // Melody length, for looping.

// Set overall tempo
long tempo = 10000;
// Set length of pause between notes
int pause = 1000;
// Loop variable to increase Rest length
int rest_count = 100; //<-BLETCHEROUS HACK; See NOTES


// PLAY TONE  ==============================================
// Pulse the speaker to play a tone for a particular duration
void playTone(int tone, long duration) {
  long elapsed_time = 0;
  if (tone > 0) { // if this isn't a Rest beat, while the tone has
    //  played less long than 'duration', pulse speaker HIGH and LOW
    while (elapsed_time < duration) {

      digitalWrite(speakerOut,HIGH);
      delayMicroseconds(tone / 2);

      // DOWN
      digitalWrite(speakerOut, LOW);
      delayMicroseconds(tone / 2);

      // Keep track of how long we pulsed
      elapsed_time += (tone);
    }
  }
  else { // Rest beat; loop times delay
    for (int j = 0; j < rest_count; j++) { // See NOTE on rest_count
      delayMicroseconds(duration);  
    }                                
  }                                
}


void setup() {
  pinMode(speakerOut, OUTPUT); // buzzer on speakerOut pin
  
  Serial.begin(9600); // start serial port output, check for same speed at Serial Monitor

  Serial.println("Start of distance measurement");

  playTone(C, 16 * tempo ); // play a C for 16 beats
}

void loop() {
  unsigned int distance = sonar.convert_cm(sonar.ping_median(5)); // read distance from ultrasonic sensor (using median of 5 iterations)
  // display value in serial monitor:
  Serial.print("Distance: ");
  Serial.println(distance);

  // something nearby?
  // sound alarm

  // end of loop
  delay(500); // wait half a second
}
